using System;
using System.Collections.Generic;
using System.Data;
using System.IO;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using OfficeOpenXml;
using MySql.Data.MySqlClient;

namespace CarTypeCheckAndSql
{
    public partial class Form1 : Form
    {
        // Ŭ  DataTable 
        private DataTable dataTable;

        // SQL   θ Ŭ  
        private string sqlSaveDirectory = @"C:\sql_statements\"; // ϴ η ϼ

        public Form1()
        {
            InitializeComponent();
            // EPPlus ̼ 
            ExcelPackage.LicenseContext = OfficeOpenXml.LicenseContext.NonCommercial;
            // DataTable ʱȭ
            dataTable = new DataTable();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            // ʿ Į DataTable ̸ ߰
            string[] requiredColumns = { "", "", "θ", "" };
            foreach (string columnName in requiredColumns)
            {
                dataTable.Columns.Add(columnName);
            }

            // ׸信  ҽ 
            dataGridView1.DataSource = dataTable;

            //    
            dataGridView1.AllowUserToAddRows = false;
            dataGridView2.AllowUserToAddRows = false;
            dataGridView3.AllowUserToAddRows = false;

            //  ʱȭ
            label4.Text = " : 0";
            label5.Text = " : 0";
            label6.Text = " : 0";
        }

        private void UpdateLabel4()
        {
            label4.Text = $" : {dataGridView1.Rows.Count}";
        }

        private void UpdateLabel5()
        {
            label5.Text = $" : {dataGridView2.Rows.Count}";
        }

        private void UpdateLabel6()
        {
            label6.Text = $" : {dataGridView3.Rows.Count}";
        }

        private void dataGridView1_DragEnter(object sender, DragEventArgs e)
        {
            HandleDragEnter(e);
        }

        private void dataGridView1_DragDrop(object sender, DragEventArgs e)
        {
            HandleDragDrop(e);
        }

        private void Form1_DragEnter(object sender, DragEventArgs e)
        {
            HandleDragEnter(e);
        }

        private void Form1_DragDrop(object sender, DragEventArgs e)
        {
            HandleDragDrop(e);
        }

        private void HandleDragEnter(DragEventArgs e)
        {
            if (e.Data.GetDataPresent(DataFormats.FileDrop))
            {
                string[] files = (string[])e.Data.GetData(DataFormats.FileDrop);
                if (files.Any(file => Path.GetExtension(file).Equals(".xlsx", StringComparison.OrdinalIgnoreCase)))
                {
                    e.Effect = DragDropEffects.Copy;
                    return;
                }
            }
            e.Effect = DragDropEffects.None;
        }

        private void HandleDragDrop(DragEventArgs e)
        {
            string[] files = (string[])e.Data.GetData(DataFormats.FileDrop);
            bool dataAdded = false;
            foreach (string file in files)
            {
                if (Path.GetExtension(file).Equals(".xlsx", StringComparison.OrdinalIgnoreCase))
                {
                    bool result = ProcessExcelFile(file);
                    if (result)
                    {
                        dataAdded = true;
                    }
                }
                else
                {
                    MessageBox.Show("xlsx ϸ ˴ϴ.", "  ", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }

            if (dataAdded)
            {
                // ߺ   ׸ 
                RemoveDuplicates();
                dataGridView1.Refresh();

                // label4 Ʈ
                UpdateLabel4();
            }
        }

        private bool ProcessExcelFile(string filePath)
        {
            try
            {
                FileInfo existingFile = new FileInfo(filePath);
                using (ExcelPackage package = new ExcelPackage(existingFile))
                {
                    ExcelWorksheet worksheet = package.Workbook.Worksheets.First();

                    // ù °  Į  
                    int startRow = 1;
                    int startCol = 1;
                    int colCount = worksheet.Dimension.End.Column;
                    int rowCount = worksheet.Dimension.End.Row;

                    // Į ̸ 
                    List<string> columns = new List<string>();
                    for (int col = startCol; col <= colCount; col++)
                    {
                        string columnName = worksheet.Cells[startRow, col].Text.Trim();
                        columns.Add(columnName);
                    }

                    // ʿ Į Ȯ
                    string[] requiredColumns = { "", "", "θ", "" };
                    bool allColumnsExist = requiredColumns.All(rc => columns.Contains(rc));

                    if (!allColumnsExist)
                    {
                        MessageBox.Show($"'{Path.GetFileName(filePath)}' Ͽ ʿ Į  ʽϴ.", "Į Ȯ", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                        return false;
                    }

                    // Į ε 
                    Dictionary<string, int> columnIndices = new Dictionary<string, int>();
                    for (int i = 0; i < columns.Count; i++)
                    {
                        columnIndices[columns[i]] = i;
                    }

                    //  ε
                    for (int row = startRow + 1; row <= rowCount; row++)
                    {
                        DataRow dataRow = dataTable.NewRow();
                        bool hasData = false;

                        foreach (string colName in requiredColumns)
                        {
                            int colIndex = columnIndices[colName] + 1; // Excel 1 
                            string cellValue = worksheet.Cells[row, colIndex].Text.Trim();

                            // θ ó
                            if (colName == "θ")
                            {
                                cellValue = ProcessDetailModel(cellValue);
                            }

                            //  ̸   ߰
                            if (colName == "")
                            {
                                cellValue = AdjustManufacturerName(cellValue);
                            }

                            dataRow[colName] = cellValue;

                            if (!string.IsNullOrEmpty(cellValue))
                            {
                                hasData = true;
                            }
                        }

                        // Ͱ ִ ุ ߰
                        if (hasData)
                        {
                            dataTable.Rows.Add(dataRow);
                        }
                    }

                    return true;
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show($"  ó   ߻߽ϴ.\n\n{ex.Message}", "", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return false;
            }
        }

        private string ProcessDetailModel(string value)
        {
            if (string.IsNullOrEmpty(value))
                return value;

            int index = value.IndexOf('(');
            if (index > 0)
            {
                // '('   ڿ '(' ')' 
                int startIndexToRemove = index - 1;
                int endIndex = value.IndexOf(')', index);
                if (endIndex > index)
                {
                    //   
                    int lengthToRemove = (endIndex - startIndexToRemove) + 1;
                    value = value.Remove(startIndexToRemove, lengthToRemove);
                }
            }
            return value;
        }

        private string RemoveAllWhitespace(string input)
        {
            return string.Concat(input.Where(c => !char.IsWhiteSpace(c)));
        }

        private void RemoveDuplicates()
        {
            // ߺ Ÿ  DataView 
            DataView view = new DataView(dataTable);
            DataTable distinctTable = view.ToTable(true, "", "", "θ", "");

            //  DataTable Ŭ  ߺ ŵ ͷ ä
            dataTable.Clear();
            foreach (DataRow row in distinctTable.Rows)
            {
                dataTable.Rows.Add(row.ItemArray);
            }
        }

        private void button2_Click(object sender, EventArgs e)
        {
            //  ʱȭ  ߰
            var result = MessageBox.Show(" ͸ Ͻðڽϱ?", " ʱȭ", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
            if (result == DialogResult.Yes)
            {
                dataTable.Clear();
                dataGridView1.DataSource = null;
                dataGridView1.DataSource = dataTable;

                // label4 Ʈ
                UpdateLabel4();

                // ٸ ׸   ʱȭ
                dataGridView2.DataSource = null;
                dataGridView3.DataSource = null;
                label5.Text = " : 0";
                label6.Text = " : 0";
            }
        }

        private void button1_Click(object sender, EventArgs e)
        {
            // IP ּ 
            string serverIp = comboBox1.Text.Trim();

            if (string.IsNullOrEmpty(serverIp))
            {
                MessageBox.Show("IP ּҸ Էּ.", "", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                return;
            }

            // dataGridView1 θ  
            List<string> detailModels = new List<string>();
            foreach (DataGridViewRow row in dataGridView1.Rows)
            {
                if (row.Cells["θ"].Value != null)
                {
                    string value = row.Cells["θ"].Value.ToString().Trim();
                    if (!string.IsNullOrEmpty(value))
                    {
                        detailModels.Add(value);
                    }
                }
            }

            // θ    ó
            if (detailModels.Count == 0)
            {
                MessageBox.Show("θ  ϴ.", "", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                return;
            }

            //  ڿ 
            string connectionString = $"Server={serverIp};Database=admin_base;Uid=root;Pwd=js4188..$!;Charset=utf8;";

            try
            {
                using (MySqlConnection conn = new MySqlConnection(connectionString))
                {
                    conn.Open();

                    // θ  Ķͷ ϴ  
                    string placeholders = string.Join(", ", detailModels.Select((s, i) => $"@detailModel{i}"));
                    string query = $"SELECT codenm2 FROM carcs WHERE mcode = 'CD' AND REPLACE(codenm2, ' ', '') IN ({placeholders})";

                    using (MySqlCommand cmd = new MySqlCommand(query, conn))
                    {
                        // Ķ ߰ ( ŵ  )
                        for (int i = 0; i < detailModels.Count; i++)
                        {
                            string valueNoSpaces = RemoveAllWhitespace(detailModels[i]);
                            cmd.Parameters.AddWithValue($"@detailModel{i}", valueNoSpaces);
                        }

                        using (MySqlDataAdapter adapter = new MySqlDataAdapter(cmd))
                        {
                            DataTable dbDataTable = new DataTable();
                            adapter.Fill(dbDataTable);

                            // ͺ̽ codenm2   Ͽ HashSet 
                            var dbDetailModels = new HashSet<string>(
                                dbDataTable.AsEnumerable()
                                    .Select(row => RemoveAllWhitespace(row.Field<string>("codenm2") ?? string.Empty))
                            );

                            // dataGridView2 dataGridView3 ǥ DataTable 
                            DataTable inDbTable = dataTable.Clone(); // dataGridView1  
                            DataTable notInDbTable = dataTable.Clone();

                            // dataGridView1   ˻Ͽ ͺ̽ ִ Ȯ
                            foreach (DataRow row in dataTable.Rows)
                            {
                                string detailModel = row.Field<string>("θ") ?? string.Empty;
                                string detailModelNoSpace = RemoveAllWhitespace(detailModel);

                                if (dbDetailModels.Contains(detailModelNoSpace))
                                {
                                    inDbTable.ImportRow(row);
                                }
                                else
                                {
                                    notInDbTable.ImportRow(row);
                                }
                            }

                            // dataGridView2 dataGridView3  ε
                            dataGridView2.DataSource = inDbTable;
                            dataGridView3.DataSource = notInDbTable;

                            // label5 label6 Ʈ
                            UpdateLabel5();
                            UpdateLabel6();
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show($"͸    ߻߽ϴ.\n\n{ex.Message}", "", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void button3_Click(object sender, EventArgs e)
        {
            //  ¥ ð ̿Ͽ ϸ 
            string timestamp = DateTime.Now.ToString("yyyyMMdd_HHmmss");
            string sqlFileName = $"insert_statements_{timestamp}.sql";
            string sqlFilePath = Path.Combine(sqlSaveDirectory, sqlFileName);

            // SQL  
            List<string> sqlStatements = GenerateSqlStatements();

            if (sqlStatements.Count == 0)
            {
                MessageBox.Show(" Ͱ ϴ.", "", MessageBoxButtons.OK, MessageBoxIcon.Information);
                return;
            }

            try
            {
                // 丮  
                if (!Directory.Exists(sqlSaveDirectory))
                {
                    Directory.CreateDirectory(sqlSaveDirectory);
                }

                // SQL  Ͽ 
                File.WriteAllLines(sqlFilePath, sqlStatements, Encoding.UTF8);

                //   Ȯ
                DialogResult result = MessageBox.Show("SQL  Ǿϴ. Ͻðڽϱ?", "SQL ", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
                if (result == DialogResult.Yes)
                {
                    // SQL  
                    ExecuteSqlStatements(sqlStatements);
                }
                else
                {
                    MessageBox.Show("SQL  Ǿ  ʾҽϴ.", "", MessageBoxButtons.OK, MessageBoxIcon.Information);
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show($"SQL     ߻߽ϴ.\n\n{ex.Message}", "", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private List<string> GenerateSqlStatements()
        {
            List<string> sqlStatements = new List<string>();

            // IP ּ  (scoden   ͺ̽  ʿ)
            string serverIp = comboBox1.Text.Trim();

            if (string.IsNullOrEmpty(serverIp))
            {
                MessageBox.Show("IP ּҸ Էּ.", "", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                return sqlStatements;
            }

            //  ڿ 
            string connectionString = $"Server={serverIp};Database=admin_base;Uid=root;Pwd=js4188..$!;Charset=utf8;";

            try
            {
                using (MySqlConnection conn = new MySqlConnection(connectionString))
                {
                    conn.Open();

                    //  mcode ִ scoden   ųʸ
                    Dictionary<string, int> scodenDict = new Dictionary<string, int>();

                    // mcode 
                    string[] mcodeList = { "CM", "CD", "CG" };

                    //  mcode ִ scoden  ȸ
                    foreach (string mcode in mcodeList)
                    {
                        int maxScoden = GetMaxScoden(conn, mcode);
                        scodenDict[mcode] = maxScoden;
                    }

                    // 귣  ( -> (scoden, scodec))
                    Dictionary<string, (int scoden, string scodec)> brandScodenDict = GetBrandScodenDict(conn);

                    //   (𵨸 + θ𵨸 ->  scoden)
                    Dictionary<string, int> modelScodenDict = new Dictionary<string, int>();

                    //   (޸ ->  scoden)
                    Dictionary<string, int> gradeScodenDict = new Dictionary<string, int>();

                    // dataGridView3 ͸  SQL  
                    foreach (DataGridViewRow row in dataGridView3.Rows)
                    {
                        if (row.IsNewRow) continue;

                        string manufacturer = row.Cells[""].Value?.ToString().Trim();
                        string modelName = row.Cells[""].Value?.ToString().Trim();
                        string detailModel = row.Cells["θ"].Value?.ToString().Trim();
                        string gradeName = row.Cells[""].Value?.ToString().Trim();

                        if (string.IsNullOrEmpty(modelName) || string.IsNullOrEmpty(gradeName) || string.IsNullOrEmpty(manufacturer))
                        {
                            continue; // ʿ Ͱ  ǳʶ
                        }

                        //  ̸    (ߺ )
                        manufacturer = AdjustManufacturerName(manufacturer);

                        // ڿ ̽ ó
                        manufacturer = EscapeString(manufacturer);
                        modelName = EscapeString(modelName);
                        detailModel = EscapeString(detailModel);
                        gradeName = EscapeString(gradeName);

                        // 귣 scoden scodec  
                        (int brandScoden, string brandScodec) brandScodes;
                        if (!brandScodenDict.TryGetValue(manufacturer, out brandScodes))
                        {
                            // 귣尡  
                            scodenDict["CM"] += 1;
                            int newBrandScoden = scodenDict["CM"];
                            string newBrandScodec = GenerateNewScodec(conn); // ο scodec 
                            string insertBrandSql = $"INSERT INTO carcs (mcode, scoden, scodec, codenm1) VALUES ('CM', {newBrandScoden}, '{newBrandScodec}', '{manufacturer}');";
                            sqlStatements.Add(insertBrandSql);

                            // 귣 ʿ ߰
                            brandScodes = (newBrandScoden, newBrandScodec);
                            brandScodenDict[manufacturer] = brandScodes;
                        }

                        //  Ű  (𵨸 + θ𵨸)
                        string modelKey = modelName + "_" + detailModel;

                        int modelScoden;
                        if (!modelScodenDict.TryGetValue(modelKey, out modelScoden))
                        {
                            //   
                            scodenDict["CD"] += 1;
                            modelScoden = scodenDict["CD"];
                            // codef1  scodec  (ڿ)
                            string insertModelSql = $"INSERT INTO carcs (mcode, scoden, scodec, codenm1, codenm2, codef1) VALUES ('CD', {modelScoden}, {modelScoden}, '{modelName}', '{detailModel}', '{brandScodes.brandScodec}');";
                            sqlStatements.Add(insertModelSql);

                            //  ʿ ߰
                            modelScodenDict[modelKey] = modelScoden;
                        }

                        //  scoden  
                        int gradeScoden;
                        if (!gradeScodenDict.TryGetValue(gradeName, out gradeScoden))
                        {
                            //   
                            scodenDict["CG"] += 1;
                            gradeScoden = scodenDict["CG"];
                            string insertGradeSql = $"INSERT INTO carcs (mcode, scoden, scodec, codenm1) VALUES ('CG', {gradeScoden}, {gradeScoden}, '{gradeName}');";
                            sqlStatements.Add(insertGradeSql);

                            //  ʿ ߰
                            gradeScodenDict[gradeName] = gradeScoden;
                        }

                        // carcs_carkind Թ 
                        string insertCarkindSql = $"INSERT INTO carcs_carkind (model, grade) VALUES ({modelScoden}, {gradeScoden});";
                        sqlStatements.Add(insertCarkindSql);
                    }
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show($"SQL     ߻߽ϴ.\n\n{ex.Message}", "", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

            return sqlStatements;
        }

        private Dictionary<string, (int scoden, string scodec)> GetBrandScodenDict(MySqlConnection conn)
        {
            Dictionary<string, (int scoden, string scodec)> brandScodenDict = new Dictionary<string, (int scoden, string scodec)>();

            string query = "SELECT codenm1, scoden, scodec FROM carcs WHERE mcode = 'CM'";
            using (MySqlCommand cmd = new MySqlCommand(query, conn))
            {
                using (MySqlDataReader reader = cmd.ExecuteReader())
                {
                    while (reader.Read())
                    {
                        string brandName = reader.GetString("codenm1");
                        //  ̸   
                        brandName = AdjustManufacturerName(brandName);
                        int scoden = reader.GetInt32("scoden");
                        string scodec = reader.GetString("scodec");
                        brandScodenDict[brandName] = (scoden, scodec);
                    }
                }
            }

            return brandScodenDict;
        }

        private int GetMaxScoden(MySqlConnection conn, string mcode)
        {
            string query = "SELECT MAX(scoden) FROM carcs WHERE mcode = @mcode";
            using (MySqlCommand cmd = new MySqlCommand(query, conn))
            {
                cmd.Parameters.AddWithValue("@mcode", mcode);
                object result = cmd.ExecuteScalar();
                if (result != DBNull.Value && result != null)
                {
                    return Convert.ToInt32(result);
                }
                else
                {
                    return 0; // ش mcode  0 
                }
            }
        }

        private string GenerateNewScodec(MySqlConnection conn)
        {
            // ο scodec   (:  ִ scodec  1 )
            string query = "SELECT MAX(CAST(scodec AS UNSIGNED)) FROM carcs WHERE mcode = 'CM'";
            using (MySqlCommand cmd = new MySqlCommand(query, conn))
            {
                object result = cmd.ExecuteScalar();
                int maxScodec = 0;
                if (result != DBNull.Value && result != null)
                {
                    int.TryParse(result.ToString(), out maxScodec);
                }
                return (maxScodec + 1).ToString();
            }
        }

        private string EscapeString(string input)
        {
            return input.Replace("'", "''");
        }

        private void ExecuteSqlStatements(List<string> sqlStatements)
        {
            // IP ּ 
            string serverIp = comboBox1.Text.Trim();

            if (string.IsNullOrEmpty(serverIp))
            {
                MessageBox.Show("IP ּҸ Էּ.", "", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                return;
            }

            //  ڿ 
            string connectionString = $"Server={serverIp};Database=admin_base;Uid=root;Pwd=js4188..$!;Charset=utf8;";

            try
            {
                using (MySqlConnection conn = new MySqlConnection(connectionString))
                {
                    conn.Open();

                    using (MySqlTransaction transaction = conn.BeginTransaction())
                    {
                        try
                        {
                            using (MySqlCommand cmd = new MySqlCommand())
                            {
                                cmd.Connection = conn;
                                cmd.Transaction = transaction;

                                foreach (string sql in sqlStatements)
                                {
                                    cmd.CommandText = sql;
                                    cmd.ExecuteNonQuery();
                                }
                            }

                            transaction.Commit();
                            MessageBox.Show("SQL   Ǿϴ.", "", MessageBoxButtons.OK, MessageBoxIcon.Information);

                            //    ٽ  
                            button1_Click(null, null);
                        }
                        catch (Exception ex)
                        {
                            transaction.Rollback();
                            MessageBox.Show($"SQL     ߻߽ϴ.\n\n{ex.Message}", "", MessageBoxButtons.OK, MessageBoxIcon.Error);
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show($"ͺ̽    ߻߽ϴ.\n\n{ex.Message}", "", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void label5_Click(object sender, EventArgs e)
        {
            // ʿ  ̺Ʈ ó ڵ ۼ
        }

        //  ̸    ޼ и
        private string AdjustManufacturerName(string manufacturer)
        {
            if (manufacturer == "ڸ(Ｚ)")
                return "Ｚ";
            else if (manufacturer == "KGƼ(ֿ)")
                return "ֿ";
            else
                return manufacturer;
        }

        private void button4_Click(object sender, EventArgs e)
        {
            try
            {
                // SQL    
                System.Diagnostics.Process.Start("explorer.exe", sqlSaveDirectory);
            }
            catch (Exception ex)
            {
                MessageBox.Show($"   ϴ.\n\n{ex.Message}", "", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

   
    }
}
